import { Component, OnInit, OnDestroy } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { Response } from '@angular/http';

import { Observable } from 'rxjs/Rx';
import { NgbActiveModal, NgbModalRef } from '@ng-bootstrap/ng-bootstrap';
import { EventManager, AlertService } from 'ng-jhipster';

import { Bottle } from './bottle.model';
import { BottlePopupService } from './bottle-popup.service';
import { BottleService } from './bottle.service';
import { Rack, RackService } from '../rack';

@Component({
    selector: 'jhi-bottle-dialog',
    templateUrl: './bottle-dialog.component.html'
})
export class BottleDialogComponent implements OnInit {

    bottle: Bottle;
    authorities: any[];
    isSaving: boolean;

    racks: Rack[];
    createdAtDp: any;

    constructor(
        public activeModal: NgbActiveModal,
        private alertService: AlertService,
        private bottleService: BottleService,
        private rackService: RackService,
        private eventManager: EventManager
    ) {
    }

    ngOnInit() {
        this.isSaving = false;
        this.authorities = ['ROLE_USER', 'ROLE_ADMIN'];
        this.rackService.query().subscribe(
            (res: Response) => { this.racks = res.json(); }, (res: Response) => this.onError(res.json()));
    }
    clear() {
        this.activeModal.dismiss('cancel');
    }

    save() {
        this.isSaving = true;
        if (this.bottle.id !== undefined) {
            this.subscribeToSaveResponse(
                this.bottleService.update(this.bottle));
        } else {
            this.subscribeToSaveResponse(
                this.bottleService.create(this.bottle));
        }
    }

    private subscribeToSaveResponse(result: Observable<Bottle>) {
        result.subscribe((res: Bottle) =>
            this.onSaveSuccess(res), (res: Response) => this.onSaveError(res));
    }

    private onSaveSuccess(result: Bottle) {
        this.eventManager.broadcast({ name: 'bottleListModification', content: 'OK'});
        this.isSaving = false;
        this.activeModal.dismiss(result);
    }

    private onSaveError(error) {
        try {
            error.json();
        } catch (exception) {
            error.message = error.text();
        }
        this.isSaving = false;
        this.onError(error);
    }

    private onError(error) {
        this.alertService.error(error.message, null, null);
    }

    trackRackById(index: number, item: Rack) {
        return item.id;
    }
}

@Component({
    selector: 'jhi-bottle-popup',
    template: ''
})
export class BottlePopupComponent implements OnInit, OnDestroy {

    modalRef: NgbModalRef;
    routeSub: any;

    constructor(
        private route: ActivatedRoute,
        private bottlePopupService: BottlePopupService
    ) {}

    ngOnInit() {
        this.routeSub = this.route.params.subscribe((params) => {
            if ( params['id'] ) {
                this.modalRef = this.bottlePopupService
                    .open(BottleDialogComponent, params['id']);
            } else {
                this.modalRef = this.bottlePopupService
                    .open(BottleDialogComponent);
            }
        });
    }

    ngOnDestroy() {
        this.routeSub.unsubscribe();
    }
}
