import { Component, OnInit, OnDestroy } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { Response } from '@angular/http';

import { Observable } from 'rxjs/Rx';
import { NgbActiveModal, NgbModalRef } from '@ng-bootstrap/ng-bootstrap';
import { EventManager, AlertService } from 'ng-jhipster';

import { Rack } from './rack.model';
import { RackPopupService } from './rack-popup.service';
import { RackService } from './rack.service';

@Component({
    selector: 'jhi-rack-dialog',
    templateUrl: './rack-dialog.component.html'
})
export class RackDialogComponent implements OnInit {

    rack: Rack;
    authorities: any[];
    isSaving: boolean;
    createdAtDp: any;

    constructor(
        public activeModal: NgbActiveModal,
        private alertService: AlertService,
        private rackService: RackService,
        private eventManager: EventManager
    ) {
    }

    ngOnInit() {
        this.isSaving = false;
        this.authorities = ['ROLE_USER', 'ROLE_ADMIN'];
    }
    clear() {
        this.activeModal.dismiss('cancel');
    }

    save() {
        this.isSaving = true;
        if (this.rack.id !== undefined) {
            this.subscribeToSaveResponse(
                this.rackService.update(this.rack));
        } else {
            this.subscribeToSaveResponse(
                this.rackService.create(this.rack));
        }
    }

    private subscribeToSaveResponse(result: Observable<Rack>) {
        result.subscribe((res: Rack) =>
            this.onSaveSuccess(res), (res: Response) => this.onSaveError(res));
    }

    private onSaveSuccess(result: Rack) {
        this.eventManager.broadcast({ name: 'rackListModification', content: 'OK'});
        this.isSaving = false;
        this.activeModal.dismiss(result);
    }

    private onSaveError(error) {
        try {
            error.json();
        } catch (exception) {
            error.message = error.text();
        }
        this.isSaving = false;
        this.onError(error);
    }

    private onError(error) {
        this.alertService.error(error.message, null, null);
    }
}

@Component({
    selector: 'jhi-rack-popup',
    template: ''
})
export class RackPopupComponent implements OnInit, OnDestroy {

    modalRef: NgbModalRef;
    routeSub: any;

    constructor(
        private route: ActivatedRoute,
        private rackPopupService: RackPopupService
    ) {}

    ngOnInit() {
        this.routeSub = this.route.params.subscribe((params) => {
            if ( params['id'] ) {
                this.modalRef = this.rackPopupService
                    .open(RackDialogComponent, params['id']);
            } else {
                this.modalRef = this.rackPopupService
                    .open(RackDialogComponent);
            }
        });
    }

    ngOnDestroy() {
        this.routeSub.unsubscribe();
    }
}
