package fr.ippon.jhipster.cellar.web.rest;

import fr.ippon.jhipster.cellar.JhipstercellarApp;

import fr.ippon.jhipster.cellar.domain.Rack;
import fr.ippon.jhipster.cellar.repository.RackRepository;
import fr.ippon.jhipster.cellar.web.rest.errors.ExceptionTranslator;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.web.PageableHandlerMethodArgumentResolver;
import org.springframework.http.MediaType;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

/**
 * Test class for the RackResource REST controller.
 *
 * @see RackResource
 */
@RunWith(SpringRunner.class)
@SpringBootTest(classes = JhipstercellarApp.class)
public class RackResourceIntTest {

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final Integer DEFAULT_NB_COLUMNS = 1;
    private static final Integer UPDATED_NB_COLUMNS = 2;

    private static final Integer DEFAULT_NB_ROWS = 1;
    private static final Integer UPDATED_NB_ROWS = 2;

    private static final String DEFAULT_IMAGE = "AAAAAAAAAA";
    private static final String UPDATED_IMAGE = "BBBBBBBBBB";

    private static final LocalDate DEFAULT_CREATED_AT = LocalDate.ofEpochDay(0L);
    private static final LocalDate UPDATED_CREATED_AT = LocalDate.now(ZoneId.systemDefault());

    @Autowired
    private RackRepository rackRepository;

    @Autowired
    private MappingJackson2HttpMessageConverter jacksonMessageConverter;

    @Autowired
    private PageableHandlerMethodArgumentResolver pageableArgumentResolver;

    @Autowired
    private ExceptionTranslator exceptionTranslator;

    @Autowired
    private EntityManager em;

    private MockMvc restRackMockMvc;

    private Rack rack;

    @Before
    public void setup() {
        MockitoAnnotations.initMocks(this);
        RackResource rackResource = new RackResource(rackRepository);
        this.restRackMockMvc = MockMvcBuilders.standaloneSetup(rackResource)
            .setCustomArgumentResolvers(pageableArgumentResolver)
            .setControllerAdvice(exceptionTranslator)
            .setMessageConverters(jacksonMessageConverter).build();
    }

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static Rack createEntity(EntityManager em) {
        Rack rack = new Rack()
            .name(DEFAULT_NAME)
            .nbColumns(DEFAULT_NB_COLUMNS)
            .nbRows(DEFAULT_NB_ROWS)
            .image(DEFAULT_IMAGE)
            .createdAt(DEFAULT_CREATED_AT);
        return rack;
    }

    @Before
    public void initTest() {
        rack = createEntity(em);
    }

    @Test
    @Transactional
    public void createRack() throws Exception {
        int databaseSizeBeforeCreate = rackRepository.findAll().size();

        // Create the Rack
        restRackMockMvc.perform(post("/api/racks")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(rack)))
            .andExpect(status().isCreated());

        // Validate the Rack in the database
        List<Rack> rackList = rackRepository.findAll();
        assertThat(rackList).hasSize(databaseSizeBeforeCreate + 1);
        Rack testRack = rackList.get(rackList.size() - 1);
        assertThat(testRack.getName()).isEqualTo(DEFAULT_NAME);
        assertThat(testRack.getNbColumns()).isEqualTo(DEFAULT_NB_COLUMNS);
        assertThat(testRack.getNbRows()).isEqualTo(DEFAULT_NB_ROWS);
        assertThat(testRack.getImage()).isEqualTo(DEFAULT_IMAGE);
        assertThat(testRack.getCreatedAt()).isEqualTo(DEFAULT_CREATED_AT);
    }

    @Test
    @Transactional
    public void createRackWithExistingId() throws Exception {
        int databaseSizeBeforeCreate = rackRepository.findAll().size();

        // Create the Rack with an existing ID
        rack.setId(1L);

        // An entity with an existing ID cannot be created, so this API call must fail
        restRackMockMvc.perform(post("/api/racks")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(rack)))
            .andExpect(status().isBadRequest());

        // Validate the Alice in the database
        List<Rack> rackList = rackRepository.findAll();
        assertThat(rackList).hasSize(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    public void getAllRacks() throws Exception {
        // Initialize the database
        rackRepository.saveAndFlush(rack);

        // Get all the rackList
        restRackMockMvc.perform(get("/api/racks?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_UTF8_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(rack.getId().intValue())))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME.toString())))
            .andExpect(jsonPath("$.[*].nbColumns").value(hasItem(DEFAULT_NB_COLUMNS)))
            .andExpect(jsonPath("$.[*].nbRows").value(hasItem(DEFAULT_NB_ROWS)))
            .andExpect(jsonPath("$.[*].image").value(hasItem(DEFAULT_IMAGE.toString())))
            .andExpect(jsonPath("$.[*].createdAt").value(hasItem(DEFAULT_CREATED_AT.toString())));
    }

    @Test
    @Transactional
    public void getRack() throws Exception {
        // Initialize the database
        rackRepository.saveAndFlush(rack);

        // Get the rack
        restRackMockMvc.perform(get("/api/racks/{id}", rack.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_UTF8_VALUE))
            .andExpect(jsonPath("$.id").value(rack.getId().intValue()))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME.toString()))
            .andExpect(jsonPath("$.nbColumns").value(DEFAULT_NB_COLUMNS))
            .andExpect(jsonPath("$.nbRows").value(DEFAULT_NB_ROWS))
            .andExpect(jsonPath("$.image").value(DEFAULT_IMAGE.toString()))
            .andExpect(jsonPath("$.createdAt").value(DEFAULT_CREATED_AT.toString()));
    }

    @Test
    @Transactional
    public void getNonExistingRack() throws Exception {
        // Get the rack
        restRackMockMvc.perform(get("/api/racks/{id}", Long.MAX_VALUE))
            .andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    public void updateRack() throws Exception {
        // Initialize the database
        rackRepository.saveAndFlush(rack);
        int databaseSizeBeforeUpdate = rackRepository.findAll().size();

        // Update the rack
        Rack updatedRack = rackRepository.findOne(rack.getId());
        updatedRack
            .name(UPDATED_NAME)
            .nbColumns(UPDATED_NB_COLUMNS)
            .nbRows(UPDATED_NB_ROWS)
            .image(UPDATED_IMAGE)
            .createdAt(UPDATED_CREATED_AT);

        restRackMockMvc.perform(put("/api/racks")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(updatedRack)))
            .andExpect(status().isOk());

        // Validate the Rack in the database
        List<Rack> rackList = rackRepository.findAll();
        assertThat(rackList).hasSize(databaseSizeBeforeUpdate);
        Rack testRack = rackList.get(rackList.size() - 1);
        assertThat(testRack.getName()).isEqualTo(UPDATED_NAME);
        assertThat(testRack.getNbColumns()).isEqualTo(UPDATED_NB_COLUMNS);
        assertThat(testRack.getNbRows()).isEqualTo(UPDATED_NB_ROWS);
        assertThat(testRack.getImage()).isEqualTo(UPDATED_IMAGE);
        assertThat(testRack.getCreatedAt()).isEqualTo(UPDATED_CREATED_AT);
    }

    @Test
    @Transactional
    public void updateNonExistingRack() throws Exception {
        int databaseSizeBeforeUpdate = rackRepository.findAll().size();

        // Create the Rack

        // If the entity doesn't have an ID, it will be created instead of just being updated
        restRackMockMvc.perform(put("/api/racks")
            .contentType(TestUtil.APPLICATION_JSON_UTF8)
            .content(TestUtil.convertObjectToJsonBytes(rack)))
            .andExpect(status().isCreated());

        // Validate the Rack in the database
        List<Rack> rackList = rackRepository.findAll();
        assertThat(rackList).hasSize(databaseSizeBeforeUpdate + 1);
    }

    @Test
    @Transactional
    public void deleteRack() throws Exception {
        // Initialize the database
        rackRepository.saveAndFlush(rack);
        int databaseSizeBeforeDelete = rackRepository.findAll().size();

        // Get the rack
        restRackMockMvc.perform(delete("/api/racks/{id}", rack.getId())
            .accept(TestUtil.APPLICATION_JSON_UTF8))
            .andExpect(status().isOk());

        // Validate the database is empty
        List<Rack> rackList = rackRepository.findAll();
        assertThat(rackList).hasSize(databaseSizeBeforeDelete - 1);
    }

    @Test
    @Transactional
    public void equalsVerifier() throws Exception {
        TestUtil.equalsVerifier(Rack.class);
        Rack rack1 = new Rack();
        rack1.setId(1L);
        Rack rack2 = new Rack();
        rack2.setId(rack1.getId());
        assertThat(rack1).isEqualTo(rack2);
        rack2.setId(2L);
        assertThat(rack1).isNotEqualTo(rack2);
        rack1.setId(null);
        assertThat(rack1).isNotEqualTo(rack2);
    }
}
